Package: localScore
Type: Package
Title: Package for Sequence Analysis by Local Score
Version: 2.0.5
Date: 2026-04-20
Authors@R: c(person("Sebastian", "Simon", role = "aut"),
            person("Chris", "Verschelden", role = "aut"),
            person("Charly", "Marty", role = "aut"),
            person("David", "Robelin", role = c("aut","cre"),
                   email = "david.robelin@inrae.fr"),
            person("Sabine", "Mercier", role = "aut",
                   email = "sabine.mercier@univ-tlse2.fr"),
            person("Sebastien", "Dejean", role = "aut",
                   email="sebastien.dejean@math.univ-toulouse.fr"),
            person(given="The authors of Eigen the library for the included version of Eigen", role = "cph"))
Copyright: See the file COPYRIGHTS for various embedded Eigen library
        copyright details
Maintainer: David Robelin <david.robelin@inrae.fr>
Description: Functionalities for calculating the local score and calculating statistical relevance (p-value) to find a local Score in a sequence of given distribution (D. Robelin, S. Déjean, S. Mercier (2025) <doi:10.24072/pcjournal.650> ; S. Mercier and J.-J. Daudin (2001) <https://hal.science/hal-00714174/>) ; S. Karlin and S. Altschul (1990) <https://pmc.ncbi.nlm.nih.gov/articles/PMC53667/>  ; S. Mercier, D. Cellier and F. Charlot (2003) <https://hal.science/hal-00937529v1/> ; A. Lagnoux, S. Mercier and P. Valois (2017) <doi:10.1093/bioinformatics/btw699> ).
License: GPL (>= 2) | file LICENSE
Imports: Rcpp (>= 0.12.16), utils
LinkingTo: Rcpp
RoxygenNote: 7.3.3
LazyData: true
Encoding: UTF-8
Suggests: knitr, rmarkdown, testthat (>= 2.1.0)
VignetteBuilder: knitr
Depends: R (>= 3.2.0)
NeedsCompilation: yes
Packaged: 2026-04-21 16:29:44 UTC; drobelin
Author: Sebastian Simon [aut],
  Chris Verschelden [aut],
  Charly Marty [aut],
  David Robelin [aut, cre],
  Sabine Mercier [aut],
  Sebastien Dejean [aut],
  The authors of Eigen the library for the included version of Eigen
    [cph]
Repository: CRAN
Date/Publication: 2026-04-21 22:22:16 UTC
Built: R 4.6.0; x86_64-apple-darwin20; 2026-04-26 21:23:27 UTC; unix
Archs: localScore.so.dSYM
