# Generated by roxygen2: do not edit by hand

export(CharSequence2ScoreSequence)
export(CharSequences2ScoreSequences)
export(RealScores2IntegerScores)
export(automatic_analysis)
export(daudin)
export(exact_mc)
export(karlin)
export(karlinMonteCarlo)
export(karlinMonteCarlo_double)
export(karlin_parameters)
export(lindley)
export(loadMatrixFromFile)
export(loadScoreFromFile)
export(localScoreC)
export(localScoreC_double)
export(localScoreC_int)
export(maxPartialSumd)
export(mcc)
export(monteCarlo)
export(monteCarlo_double)
export(proba_theoretical_first_excursion_iid)
export(proba_theoretical_ith_excursion_iid)
export(proba_theoretical_ith_excursion_markov)
export(recordTimes)
export(scoreSequences2probabilityVector)
export(sequences2transmatrix)
export(stationary_distribution)
export(transmatrix2sequence)
importFrom(Rcpp,evalCpp)
importFrom(graphics,abline)
importFrom(graphics,barplot)
importFrom(graphics,par)
importFrom(graphics,plot)
importFrom(stats,density)
importFrom(stats,ecdf)
importFrom(stats,knots)
importFrom(stats,lm)
importFrom(utils,read.csv)
importFrom(utils,setTxtProgressBar)
importFrom(utils,txtProgressBar)
useDynLib(localScore)
